! This source code file was last time modified by Igor UA3DJY on January 23rd, 2018
! All changes are shown in the patch file coming together with the full JTDX source code.

module jt65_mod6

  integer, parameter :: NPTS=624000
  real*4 dd(NPTS)
  integer nnmode
  save

end module jt65_mod6